<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class BarberBlock extends Model
{
    use HasFactory;

    protected $fillable = [
        'barber_id',
        'kind',
        'block_date',
        'start_time',
        'end_time',
        'reason',
    ];

    public function barber()
    {
        return $this->belongsTo(Barber::class);
    }
}
