import{_ as d,u as c,o as b,c as p,b as a,t as u,d as v,h as t,D as m,i as r,l as n,A as i,j as f}from"./index-D0xspOra.js";import{L as _}from"./LogoutButton-BLrpUxw8.js";const B={class:"barber-layout"},h={class:"barber-header"},g={class:"header-title"},C={class:"barber-header-actions"},$={class:"barber-content"},k={class:"bottom-nav"},y={__name:"BarberLayout",setup(A){const l=c();return(s,e)=>{const o=f("router-link");return b(),p("div",B,[a("header",h,[a("div",null,[e[0]||(e[0]=a("div",{class:"header-mode"},"Modo Barbero",-1)),a("div",g,u(v(l).user?.name?.toUpperCase()||"BARBERO"),1)]),a("div",C,[t(_),e[1]||(e[1]=a("span",{class:"badge badge-green"},"Activo",-1))])]),a("main",$,[m(s.$slots,"default",{},void 0)]),a("nav",k,[t(o,{to:"/barber/agenda",class:i(["nav-item",{active:s.$route.path==="/barber/agenda"}])},{default:r(()=>[...e[2]||(e[2]=[a("span",{class:"nav-icon"},"📋",-1),n(" Turnos ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/clientes",class:i(["nav-item",{active:s.$route.path==="/barber/clientes"}])},{default:r(()=>[...e[3]||(e[3]=[a("span",{class:"nav-icon"},"👥",-1),n(" Clientes ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/servicio-especial",class:i(["nav-item",{active:s.$route.path==="/barber/servicio-especial"}])},{default:r(()=>[...e[4]||(e[4]=[a("span",{class:"nav-icon"},"✂️",-1),n(" Especial ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/perfil",class:i(["nav-item",{active:s.$route.path==="/barber/perfil"}])},{default:r(()=>[...e[5]||(e[5]=[a("span",{class:"nav-icon"},"👤",-1),n(" Perfil ",-1)])]),_:1},8,["class"])])])}}},E=d(y,[["__scopeId","data-v-f1e0aaf1"]]);export{E as B};
