.agenda-header[data-v-fe8e39bd]{margin-bottom:20px}.date-input[data-v-fe8e39bd]{max-width:200px}.no-appointments[data-v-fe8e39bd]{text-align:center;padding:60px 20px;color:var(--text-muted)}.appointments-list[data-v-fe8e39bd]{display:flex;flex-direction:column;gap:12px}.appointment-card[data-v-fe8e39bd]{padding:16px}.appointment-card.card-progress[data-v-fe8e39bd]{border-left:4px solid var(--success)}.appointment-card.card-next[data-v-fe8e39bd]{border-left:4px solid var(--gold)}.appointment-card.card-pending[data-v-fe8e39bd],.appointment-card.card-muted[data-v-fe8e39bd]{opacity:.6}.card-top[data-v-fe8e39bd]{display:flex;justify-content:space-between;align-items:center;margin-bottom:10px}.card-time[data-v-fe8e39bd]{color:var(--gold)}.client-name[data-v-fe8e39bd]{font-size:18px;display:block;margin-bottom:4px}.service-name[data-v-fe8e39bd]{color:var(--text-muted);font-size:14px}.card-actions[data-v-fe8e39bd]{display:flex;flex-wrap:wrap;gap:8px;margin-top:12px}.card-actions .btn[data-v-fe8e39bd]{min-height:44px;flex:1;min-width:120px}
