.month-pick[data-v-e348e4ce]{width:100%;max-width:100%}.month-pick--embed .fc-wrap[data-v-e348e4ce]{box-shadow:0 4px 24px #00000059}.month-pick__toggle[data-v-e348e4ce]{display:flex;align-items:center;justify-content:space-between;width:100%;margin:0 0 10px;padding:10px 12px;border:1px solid var(--border);border-radius:10px;background:var(--bg-device);color:var(--text-main);font:inherit;font-size:14px;font-weight:700;cursor:pointer;text-align:left;touch-action:manipulation}.month-pick__toggle[data-v-e348e4ce]:hover{border-color:color-mix(in srgb,var(--gold) 35%,var(--border))}.month-pick__chev[data-v-e348e4ce]{display:inline-block;transition:transform .2s ease;color:var(--gold);font-size:1rem}.month-pick__chev.is-open[data-v-e348e4ce]{transform:rotate(180deg)}.fc-wrap[data-v-e348e4ce]{border-radius:12px;border:1px solid var(--border);overflow:hidden;background:var(--bg-page);min-height:280px}.fc-wrap[data-v-e348e4ce] .fc{--fc-border-color: var(--border);--fc-page-bg-color: var(--bg-page);--fc-neutral-bg-color: var(--bg-device);--fc-today-bg-color: rgba(212, 175, 55, .12);--fc-button-bg-color: var(--gold);--fc-button-border-color: transparent;--fc-button-text-color: #111;font-size:13px}.fc-wrap[data-v-e348e4ce] .fc-view-harness{min-height:240px}.fc-wrap[data-v-e348e4ce] .fc-scrollgrid{border-color:var(--border)!important}.fc-wrap[data-v-e348e4ce] .fc-col-header-cell{padding:10px 2px;font-size:.72rem;font-weight:700;text-transform:uppercase;letter-spacing:.04em;color:var(--text-muted)}.fc-wrap[data-v-e348e4ce] .fc-toolbar-title{font-size:1.05rem;font-weight:700;color:var(--text-main)}.fc-wrap[data-v-e348e4ce] .fc-button{padding:.45em .65em;font-size:.85rem;border-radius:8px;min-height:44px;min-width:44px;touch-action:manipulation}.fc-wrap[data-v-e348e4ce] .fc-button-primary{background:var(--gold)!important;border-color:transparent!important;color:#111!important;font-weight:600}.fc-wrap[data-v-e348e4ce] .fc-button-primary:not(:disabled):active,.fc-wrap[data-v-e348e4ce] .fc-button-primary:not(:disabled):hover{filter:brightness(.95)}.fc-wrap[data-v-e348e4ce] .fc-toolbar.fc-header-toolbar{margin:.65rem .5rem;flex-wrap:wrap;gap:8px}.fc-wrap[data-v-e348e4ce] .fc-toolbar-chunk{display:flex;align-items:center;flex-wrap:wrap;gap:6px}@media(max-width:520px){.fc-wrap[data-v-e348e4ce] .fc-toolbar.fc-header-toolbar{flex-direction:column;align-items:stretch;margin:.5rem .35rem}.fc-wrap[data-v-e348e4ce] .fc-toolbar-chunk:nth-child(2){order:-1;justify-content:center}.fc-wrap[data-v-e348e4ce] .fc-toolbar-chunk:first-child,.fc-wrap[data-v-e348e4ce] .fc-toolbar-chunk:last-child{justify-content:center}.fc-wrap[data-v-e348e4ce] .fc-toolbar-title{font-size:1rem}}.fc-wrap[data-v-e348e4ce] .fc-daygrid-day-top{flex-direction:row;justify-content:center;padding-top:4px}.fc-wrap[data-v-e348e4ce] .fc-daygrid-day-number{padding:6px 8px;border-radius:8px;font-weight:600;font-size:.95rem}.fc-wrap[data-v-e348e4ce] .fc-daygrid-day-frame{min-height:2.85rem}@media(min-width:400px){.fc-wrap[data-v-e348e4ce] .fc-daygrid-day-frame{min-height:3.1rem}}.fc-wrap[data-v-e348e4ce] .fc-col-header-cell,.fc-wrap[data-v-e348e4ce] .fc-daygrid-day-number{color:var(--text-main)}.fc-wrap[data-v-e348e4ce] .cal-day-past .fc-daygrid-day-number{opacity:.35}.fc-wrap[data-v-e348e4ce] .cal-day-has-slots{background:#16a34a2e!important}.fc-wrap[data-v-e348e4ce] .cal-day-has-slots .fc-daygrid-day-number{color:var(--success)}.fc-wrap[data-v-e348e4ce] .cal-day-selected{outline:none;background:transparent!important}.fc-wrap[data-v-e348e4ce] .cal-day-selected .fc-daygrid-day-number{box-shadow:0 0 0 2px var(--gold);background:color-mix(in srgb,var(--gold) 26%,transparent);color:var(--text-main)}.fc-wrap[data-v-e348e4ce] .cal-appt-badge{display:inline-flex;align-items:center;justify-content:center;min-width:18px;height:18px;padding:0 5px;margin-left:6px;border-radius:999px;font-size:10px;font-weight:800;background:var(--gold);color:#111}
