import{_ as r,o as c,c as l,b as i,l as p,t as d,D as u,z as m}from"./index-2dimE3JC.js";const f={__name:"StatusBadge",props:{status:{type:String,required:!0}},setup(e){const t=e,n=m(()=>({pending:"Pendiente",confirmed:"Confirmado",completed:"Completado",cancelled:"Cancelado"})[t.status]||t.status);return(a,s)=>(c(),l("span",{class:u(["badge",`badge-${t.status}`])},[s[0]||(s[0]=i("span",{class:"badge-dot"},null,-1)),p(" "+d(n.value),1)],2))}},_=r(f,[["__scopeId","data-v-e5e2286e"]]);function b(e){if(e==null||e==="")return"";const t=String(e).trim();if(/^https?:\/\//i.test(t))return t;const n="/api";try{const a=new URL(n||(typeof window<"u"?window.location.href:"http://localhost/")),s=a.origin+a.pathname.replace(/\/?api\/?$/,""),o=t.startsWith("/")?t:`/${t}`;return`${s}${o}`}catch{return t}}export{_ as S,b as r};
