.client-home[data-v-5a549fc3]{display:flex;flex-direction:column;gap:10px}.card--tight[data-v-5a549fc3]{padding:12px 14px;margin-bottom:0}.exp-head[data-v-5a549fc3]{margin-bottom:.35rem}.exp-head__row[data-v-5a549fc3]{display:flex;justify-content:space-between;align-items:flex-start;gap:12px}.exp-head__eyebrow[data-v-5a549fc3]{font-size:.72rem;font-weight:700;letter-spacing:.12em;text-transform:uppercase;color:var(--gold);margin:0 0 .35rem;opacity:.9}.exp-head__title[data-v-5a549fc3]{margin:0 0 .35rem;font-size:1.4rem;font-weight:800;letter-spacing:-.02em;line-height:1.2}.exp-head__lead[data-v-5a549fc3]{margin:0;font-size:.88rem;line-height:1.42;color:var(--text-muted)}.exp-head__refresh[data-v-5a549fc3]{flex-shrink:0}.home-reserve[data-v-5a549fc3]{margin-bottom:0}.home-reserve__btn[data-v-5a549fc3]{display:flex;align-items:center;justify-content:center;width:100%;min-height:48px;padding:12px 18px;border-radius:10px;font-weight:700;font-size:1rem;text-decoration:none;box-sizing:border-box}.exp-section-title[data-v-5a549fc3]{margin:0 0 .25rem;font-size:.78rem;font-weight:800;letter-spacing:.08em;text-transform:uppercase;color:var(--gold)}.exp-section-desc[data-v-5a549fc3]{margin:0 0 .5rem;font-size:.8rem;color:var(--text-muted);line-height:1.35}.exp-timeline[data-v-5a549fc3]{margin-bottom:0}.exp-list[data-v-5a549fc3]{list-style:none;margin:0;padding:0;display:flex;flex-direction:column;gap:6px}.exp-row-btn[data-v-5a549fc3]{width:100%;min-height:44px;display:flex;flex-wrap:wrap;align-items:center;justify-content:space-between;gap:8px;padding:10px;background:var(--bg-page);border-radius:10px;border:1px solid var(--border);cursor:pointer;font:inherit;color:inherit;text-align:left;transition:background .15s ease;touch-action:manipulation}.exp-row-btn[data-v-5a549fc3]:hover{background:color-mix(in srgb,var(--bg-card) 90%,var(--bg-page))}.exp-row-btn[data-v-5a549fc3]:focus-visible{outline:2px solid var(--gold);outline-offset:2px}.exp-row__main[data-v-5a549fc3]{display:flex;flex-wrap:wrap;gap:6px 12px;font-size:.88rem;color:var(--text-main);min-width:0}.exp-row__date[data-v-5a549fc3]{font-weight:700;color:var(--gold)}.exp-row__time[data-v-5a549fc3]{color:var(--text-muted)}.exp-row__barber[data-v-5a549fc3]{color:var(--text-muted);font-size:.82rem;width:100%}.exp-no-apt[data-v-5a549fc3]{margin:0;font-size:.9rem;color:var(--text-muted)}.exp-carousel-section[data-v-5a549fc3]{margin-top:0}.exp-carousel__track[data-v-5a549fc3]{display:flex;gap:12px;overflow-x:auto;scroll-snap-type:x mandatory;-webkit-overflow-scrolling:touch;padding-bottom:8px;margin:0 -4px;padding-left:4px;padding-right:4px}.exp-carousel__slide[data-v-5a549fc3]{flex:0 0 min(78vw,260px);scroll-snap-align:start}.exp-carousel__img-btn[data-v-5a549fc3]{display:block;width:100%;padding:0;border:none;border-radius:10px;overflow:hidden;border:1px solid var(--border);aspect-ratio:3 / 4;background:#000;cursor:pointer;touch-action:manipulation}.exp-carousel__img-btn img[data-v-5a549fc3]{width:100%;height:100%;object-fit:cover;display:block}.exp-carousel__meta[data-v-5a549fc3]{display:flex;flex-direction:column;gap:2px;font-size:11px;color:var(--text-muted);margin-top:6px;line-height:1.3}.exp-empty--carousel[data-v-5a549fc3]{text-align:center;padding:1.1rem 1rem;margin-top:0}.exp-empty__title[data-v-5a549fc3]{font-size:1rem;font-weight:700;margin:0 0 .5rem}.exp-empty__text[data-v-5a549fc3]{margin:0;font-size:.9rem;color:var(--text-muted);line-height:1.5}.exp-lightbox[data-v-5a549fc3]{position:fixed;inset:0;z-index:10100;background:#000000eb;display:flex;align-items:center;justify-content:center;padding:16px;padding-top:env(safe-area-inset-top);padding-bottom:env(safe-area-inset-bottom)}.exp-lightbox__toolbar[data-v-5a549fc3]{position:absolute;top:calc(12px + env(safe-area-inset-top));left:16px;z-index:2}.exp-lightbox__dl[data-v-5a549fc3]{min-height:40px;padding:8px 14px;font-size:13px}.exp-lightbox__close[data-v-5a549fc3]{position:absolute;top:calc(12px + env(safe-area-inset-top));right:16px;width:44px;height:44px;border:none;border-radius:999px;background:color-mix(in srgb,var(--bg-card) 80%,transparent);color:var(--text-main);font-size:1.75rem;line-height:1;cursor:pointer;touch-action:manipulation;z-index:1}.exp-lightbox__figure[data-v-5a549fc3]{margin:0;max-width:100%;max-height:100%;display:flex;flex-direction:column;align-items:center;gap:12px}.exp-lightbox__figure img[data-v-5a549fc3]{max-width:100%;max-height:min(78vh,900px);width:auto;height:auto;object-fit:contain;border-radius:8px}.exp-lightbox__cap[data-v-5a549fc3]{font-size:.88rem;color:var(--text-muted);text-align:center;max-width:28rem;line-height:1.45}
