import{_ as f,u as k,o as d,c as m,a as e,q as n,f as i,g as u,j as r,x as C,r as $,h as c,i as y}from"./index-oOcXnyQ9.js";const g={class:"admin-layout"},A={key:0},S={key:1},j={class:"sidebar-nav"},B={class:"admin-main"},L={__name:"AdminLayout",setup(N){const v=y(),p=k(),a=$(!1),b=async()=>{await p.logout(),v.push("/login")};return(t,s)=>{const o=c("router-link");return d(),m("div",g,[e("button",{class:"sidebar-toggle",onClick:s[0]||(s[0]=l=>a.value=!a.value),"aria-label":"Menú"},[a.value?(d(),m("span",S,"✕")):(d(),m("span",A,"☰"))]),e("div",{class:n(["sidebar-backdrop",{open:a.value}]),onClick:s[1]||(s[1]=l=>a.value=!1)},null,2),e("aside",{class:n(["sidebar",{open:a.value}])},[s[12]||(s[12]=e("div",{class:"sidebar-brand"},"San Ramón",-1)),e("nav",j,[i(o,{to:"/admin",class:n(["menu-item",{active:t.$route.path==="/admin"}]),onClick:s[2]||(s[2]=l=>a.value=!1)},{default:u(()=>[...s[7]||(s[7]=[e("span",null,"📊",-1),r(" Monitor Hoy ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/barbers",class:n(["menu-item",{active:t.$route.path==="/admin/barbers"}]),onClick:s[3]||(s[3]=l=>a.value=!1)},{default:u(()=>[...s[8]||(s[8]=[e("span",null,"💈",-1),r(" Barberos ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/services",class:n(["menu-item",{active:t.$route.path==="/admin/services"}]),onClick:s[4]||(s[4]=l=>a.value=!1)},{default:u(()=>[...s[9]||(s[9]=[e("span",null,"✂️",-1),r(" Servicios ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/clients",class:n(["menu-item",{active:t.$route.path==="/admin/clients"}]),onClick:s[5]||(s[5]=l=>a.value=!1)},{default:u(()=>[...s[10]||(s[10]=[e("span",null,"👥",-1),r(" Clientes ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/ajustes",class:n(["menu-item",{active:t.$route.path==="/admin/ajustes"}]),onClick:s[6]||(s[6]=l=>a.value=!1)},{default:u(()=>[...s[11]||(s[11]=[e("span",null,"⚙️",-1),r(" Ajustes App ",-1)])]),_:1},8,["class"])]),e("div",{class:"sidebar-footer"},[e("button",{onClick:b,class:"btn btn-outline btn-sm"},"Cerrar Sesión")])],2),e("main",B,[C(t.$slots,"default",{},void 0)])])}}},w=f(L,[["__scopeId","data-v-ad5c4775"]]);export{w as A};
