function s(a,n=""){const t=String(a??"").replace(/\D/g,"");if(t.length<8||t.length>15)return"";const e=`https://wa.me/${t}`;if(n){const r=new URLSearchParams;return r.set("text",n),`${e}?${r.toString()}`}return e}export{s as b};
