import{_ as d,o as s,d as r,c as l,a,t as c,e as n,x as m,T as u}from"./index-oOcXnyQ9.js";const f={key:0,class:"modal-overlay"},b={class:"modal-card"},k={key:0,class:"modal-title"},v={key:1,class:"modal-actions"},y={__name:"ModalDialog",props:{show:Boolean,title:String,showActions:{type:Boolean,default:!0}},emits:["close","confirm"],setup(o){return(e,t)=>(s(),r(u,{to:"body"},[o.show?(s(),l("div",f,[a("div",{class:"modal-backdrop",onClick:t[0]||(t[0]=i=>e.$emit("close"))}),a("div",b,[o.title?(s(),l("h3",k,c(o.title),1)):n("",!0),m(e.$slots,"default",{},void 0),o.showActions?(s(),l("div",v,[a("button",{onClick:t[1]||(t[1]=i=>e.$emit("close")),class:"btn btn-outline btn-sm"}," Cancelar "),a("button",{onClick:t[2]||(t[2]=i=>e.$emit("confirm")),class:"btn btn-gold btn-sm"}," Confirmar ")])):n("",!0)])])):n("",!0)]))}},p=d(y,[["__scopeId","data-v-f9f8c688"]]);export{p as M};
