.modal-overlay[data-v-f34b211b]{position:fixed;inset:0;z-index:1000;display:flex;flex-direction:column;align-items:center;justify-content:center;padding:max(12px,env(safe-area-inset-top)) 16px max(12px,env(safe-area-inset-bottom));overflow-y:auto;overflow-x:hidden;-webkit-overflow-scrolling:touch;overscroll-behavior:contain}.modal-backdrop[data-v-f34b211b]{position:absolute;inset:0;background:#000000d9;-webkit-backdrop-filter:blur(4px);backdrop-filter:blur(4px)}.modal-card[data-v-f34b211b]{position:relative;z-index:1;display:flex;flex-direction:column;max-height:min(92dvh,calc(100vh - 24px));background:var(--bg-card);border:1px solid var(--border);border-radius:12px;padding:20px 20px 16px;max-width:min(420px,100%);width:100%;animation:modalIn-f34b211b .2s ease;margin:auto;flex-shrink:0;overflow:hidden;box-sizing:border-box}.modal-body[data-v-f34b211b]{flex:1 1 auto;min-height:0;overflow-x:hidden;overflow-y:auto;-webkit-overflow-scrolling:touch;overscroll-behavior:contain}.modal-card--wide[data-v-f34b211b]{max-width:min(560px,100%)}.modal-card--extra-wide[data-v-f34b211b]{max-width:min(900px,100%)}@media(max-width:480px){.modal-card[data-v-f34b211b]{padding:16px 14px 12px;max-height:min(94dvh,calc(100vh - 16px))}}@keyframes modalIn-f34b211b{0%{opacity:0;transform:scale(.95)}to{opacity:1;transform:scale(1)}}.modal-title[data-v-f34b211b]{flex-shrink:0;font-size:17px;font-weight:800;color:var(--gold);margin:0 0 14px;padding-right:4px;letter-spacing:.06em;line-height:1.25}.modal-actions[data-v-f34b211b]{flex-shrink:0;display:flex;flex-wrap:wrap;justify-content:flex-end;gap:10px;margin-top:16px;padding-top:14px;border-top:1px solid color-mix(in srgb,var(--border) 85%,transparent)}
