.client-stack[data-v-77ae62fe]{display:flex;flex-direction:column;gap:10px}.card--tight[data-v-77ae62fe]{padding:12px 14px;margin-bottom:0}.cita-eyebrow[data-v-77ae62fe]{margin:0 0 8px;font-size:.72rem;font-weight:700;letter-spacing:.1em;text-transform:uppercase;color:var(--gold);opacity:.95}.cita-cancel-full[data-v-77ae62fe]{width:100%;min-height:48px;touch-action:manipulation}.cita-card__line[data-v-77ae62fe]{margin:0 0 8px;font-size:.95rem}.cita-card__line.muted[data-v-77ae62fe]{color:var(--text-muted);font-size:.88rem}.cita-card__badge[data-v-77ae62fe]{margin:12px 0 8px}.cita-card__notes[data-v-77ae62fe]{margin:12px 0 0;font-size:.88rem;line-height:1.45}.cita-card__label[data-v-77ae62fe]{display:block;font-size:.72rem;text-transform:uppercase;letter-spacing:.06em;color:var(--text-muted);margin-bottom:4px}.cita-carousel-title[data-v-77ae62fe]{margin:0 0 6px;font-size:.78rem;font-weight:800;letter-spacing:.08em;text-transform:uppercase;color:var(--gold)}.cita-carousel-hint[data-v-77ae62fe]{margin:0 0 12px;font-size:.82rem;color:var(--text-muted);line-height:1.4}.cita-no-ph[data-v-77ae62fe]{padding:1rem;font-size:.9rem;color:var(--text-muted);margin:0}.modal-text[data-v-77ae62fe]{margin:0;font-size:15px;color:var(--text-main)}.exp-carousel__track[data-v-77ae62fe]{display:flex;gap:12px;overflow-x:auto;scroll-snap-type:x mandatory;-webkit-overflow-scrolling:touch;padding-bottom:6px;margin:0 -4px;padding-left:4px;padding-right:4px}.exp-carousel__slide[data-v-77ae62fe]{flex:0 0 min(78vw,260px);scroll-snap-align:start}.exp-carousel__img-btn[data-v-77ae62fe]{display:block;width:100%;padding:0;border:none;border-radius:10px;overflow:hidden;border:1px solid var(--border);aspect-ratio:3 / 4;background:#000;cursor:pointer;touch-action:manipulation}.exp-carousel__img-btn img[data-v-77ae62fe]{width:100%;height:100%;object-fit:cover;display:block}.exp-lightbox[data-v-77ae62fe]{position:fixed;inset:0;z-index:10100;background:#000000eb;display:flex;align-items:center;justify-content:center;padding:16px;padding-top:env(safe-area-inset-top);padding-bottom:env(safe-area-inset-bottom)}.exp-lightbox__toolbar[data-v-77ae62fe]{position:absolute;top:calc(12px + env(safe-area-inset-top));left:16px;z-index:2}.exp-lightbox__dl[data-v-77ae62fe]{min-height:40px;padding:8px 14px;font-size:13px}.exp-lightbox__close[data-v-77ae62fe]{position:absolute;top:calc(12px + env(safe-area-inset-top));right:16px;width:44px;height:44px;border:none;border-radius:999px;background:color-mix(in srgb,var(--bg-card) 80%,transparent);color:var(--text-main);font-size:1.75rem;line-height:1;cursor:pointer;touch-action:manipulation;z-index:1}.exp-lightbox__figure[data-v-77ae62fe]{margin:0;max-width:100%;max-height:100%;display:flex;flex-direction:column;align-items:center;gap:12px}.exp-lightbox__figure img[data-v-77ae62fe]{max-width:100%;max-height:min(78vh,900px);width:auto;height:auto;object-fit:contain;border-radius:8px}.exp-lightbox__cap[data-v-77ae62fe]{font-size:.88rem;color:var(--text-muted);text-align:center;max-width:28rem;line-height:1.45}
