<?php

declare(strict_types=1);

namespace SpomkyLabs\Pki\ASN1\Type\Tagged;

/**
 * Intermediate class to store DER data of a private tagging type.
 */
final class PrivateType extends DERTaggedType
{
}
