import{_ as n,o as r,c as l,b as c,l as d,t as p,B as u,y as i}from"./index-DvZwo3wX.js";const m={__name:"StatusBadge",props:{status:{type:String,required:!0}},setup(s){const e=s,t=i(()=>({pending:"Pendiente",confirmed:"Confirmado",completed:"Completado",cancelled:"Cancelado"})[e.status]||e.status);return(o,a)=>(r(),l("span",{class:u(["badge",`badge-${e.status}`])},[a[0]||(a[0]=c("span",{class:"badge-dot"},null,-1)),d(" "+p(t.value),1)],2))}},g=n(m,[["__scopeId","data-v-e5e2286e"]]);export{g as S};
