.intro[data-v-2b42ff35]{font-size:13px;color:var(--text-muted);line-height:1.45;margin:0 0 20px}.empty-card[data-v-2b42ff35]{padding:24px;color:var(--text-muted)}.table-wrap[data-v-2b42ff35]{padding:0;overflow-x:auto;-webkit-overflow-scrolling:touch}.data-table[data-v-2b42ff35]{width:100%;border-collapse:collapse;font-size:14px}.data-table th[data-v-2b42ff35],.data-table td[data-v-2b42ff35]{padding:12px 14px;text-align:left;border-bottom:1px solid var(--border)}.data-table th[data-v-2b42ff35]{font-size:11px;text-transform:uppercase;letter-spacing:.06em;color:var(--text-muted);font-weight:600}.data-table tbody tr[data-v-2b42ff35]:hover{background:color-mix(in srgb,var(--bg-card) 88%,transparent)}
