.modal-overlay[data-v-cb858ffb]{position:fixed;inset:0;z-index:1000;display:flex;flex-direction:column;align-items:center;justify-content:center;padding:max(12px,env(safe-area-inset-top)) 16px max(12px,env(safe-area-inset-bottom));overflow-y:auto;overflow-x:hidden;-webkit-overflow-scrolling:touch;overscroll-behavior:contain}.modal-overlay--sheet[data-v-cb858ffb]{align-items:stretch;justify-content:flex-end;padding:0}.modal-backdrop[data-v-cb858ffb]{position:absolute;inset:0;background:#000000d9;-webkit-backdrop-filter:blur(4px);backdrop-filter:blur(4px)}.modal-card[data-v-cb858ffb]{position:relative;z-index:1;display:flex;flex-direction:column;max-height:min(92dvh,calc(100vh - 24px));background:var(--bg-card);border:1px solid var(--border);border-radius:12px;padding:20px 20px 16px;max-width:min(420px,100%);width:100%;animation:modalIn-cb858ffb .2s ease;margin:auto;flex-shrink:0;overflow:hidden;box-sizing:border-box}.modal-body[data-v-cb858ffb]{flex:1 1 auto;min-height:0;overflow-x:hidden;overflow-y:auto;-webkit-overflow-scrolling:touch;overscroll-behavior:contain}.modal-card--wide[data-v-cb858ffb]{max-width:min(560px,100%)}.modal-card--extra-wide[data-v-cb858ffb]{max-width:min(900px,100%)}@media(max-width:480px){.modal-card[data-v-cb858ffb]:not(.modal-card--sheet){padding:16px 14px 12px;max-height:min(94dvh,calc(100vh - 16px))}}.modal-card--sheet[data-v-cb858ffb]{max-width:none;width:100%;max-height:min(92dvh,calc(100vh - env(safe-area-inset-bottom)));margin:0;border-radius:16px 16px 0 0;padding:12px 16px max(16px,env(safe-area-inset-bottom));animation:sheetIn-cb858ffb .28s cubic-bezier(.22,1,.36,1);border-left:none;border-right:none;border-bottom:none}.modal-overlay--sheet .modal-backdrop[data-v-cb858ffb]{-webkit-backdrop-filter:blur(6px);backdrop-filter:blur(6px)}.modal-card--sheet .modal-actions[data-v-cb858ffb]{flex-wrap:nowrap}.modal-card--sheet .modal-actions .btn[data-v-cb858ffb]{min-height:44px;padding-left:18px;padding-right:18px}.modal-sheet-handle[data-v-cb858ffb]{width:36px;height:4px;border-radius:999px;background:color-mix(in srgb,var(--text-muted) 45%,var(--border));margin:0 auto 12px;flex-shrink:0}@keyframes modalIn-cb858ffb{0%{opacity:0;transform:scale(.95)}to{opacity:1;transform:scale(1)}}@keyframes sheetIn-cb858ffb{0%{opacity:0;transform:translateY(100%)}to{opacity:1;transform:translateY(0)}}.modal-title[data-v-cb858ffb]{flex-shrink:0;font-size:17px;font-weight:800;color:var(--gold);margin:0 0 14px;padding-right:4px;letter-spacing:.06em;line-height:1.25}.modal-actions[data-v-cb858ffb]{flex-shrink:0;display:flex;flex-wrap:wrap;justify-content:flex-end;gap:10px;margin-top:16px;padding-top:14px;border-top:1px solid color-mix(in srgb,var(--border) 85%,transparent)}
