<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    public function up(): void
    {
        Schema::create('mobile_expo_push_tokens', function (Blueprint $table) {
            $table->id();
            $table->foreignId('user_id')->constrained()->cascadeOnDelete();
            $table->string('expo_token', 512);
            $table->string('device_id', 191)->nullable();
            $table->timestamps();

            $table->unique(['user_id', 'expo_token']);
        });
    }

    public function down(): void
    {
        Schema::dropIfExists('mobile_expo_push_tokens');
    }
};
