.month-pick[data-v-e3db4d28]{width:100%}.month-pick__toggle[data-v-e3db4d28]{display:flex;align-items:center;justify-content:space-between;width:100%;margin:0 0 10px;padding:10px 12px;border:1px solid var(--border);border-radius:10px;background:var(--bg-device);color:var(--text-main);font:inherit;font-size:14px;font-weight:700;cursor:pointer;text-align:left;touch-action:manipulation}.month-pick__toggle[data-v-e3db4d28]:hover{border-color:color-mix(in srgb,var(--gold) 35%,var(--border))}.month-pick__chev[data-v-e3db4d28]{display:inline-block;transition:transform .2s ease;color:var(--gold);font-size:1rem}.month-pick__chev.is-open[data-v-e3db4d28]{transform:rotate(180deg)}.fc-wrap[data-v-e3db4d28]{border-radius:12px;border:1px solid var(--border);overflow:hidden;background:var(--bg-page)}.fc-wrap[data-v-e3db4d28] .fc{--fc-border-color: var(--border);--fc-page-bg-color: var(--bg-page);--fc-neutral-bg-color: var(--bg-device);--fc-today-bg-color: rgba(212, 175, 55, .1);font-size:13px}.fc-wrap[data-v-e3db4d28] .fc-toolbar-title{font-size:1.05rem;font-weight:700;color:var(--text-main)}.fc-wrap[data-v-e3db4d28] .fc-button-primary{background:var(--gold)!important;border-color:transparent!important;color:#111!important;font-weight:600}.fc-wrap[data-v-e3db4d28] .fc-col-header-cell,.fc-wrap[data-v-e3db4d28] .fc-daygrid-day-number{color:var(--text-main)}.fc-wrap[data-v-e3db4d28] .cal-day-past .fc-daygrid-day-number{opacity:.35}.fc-wrap[data-v-e3db4d28] .cal-day-has-slots{background:#16a34a24!important}.fc-wrap[data-v-e3db4d28] .cal-day-selected{outline:2px solid var(--gold);outline-offset:-2px}.fc-wrap[data-v-e3db4d28] .cal-appt-badge{display:inline-flex;align-items:center;justify-content:center;min-width:18px;height:18px;padding:0 5px;margin-left:6px;border-radius:999px;font-size:10px;font-weight:800;background:var(--gold);color:#111}
