<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    public function up(): void
    {
        Schema::table('appointments', function (Blueprint $table) {
            $table->timestamp('reminder_client_push_sent_at')->nullable()->after('notes');
            $table->timestamp('reminder_barber_push_sent_at')->nullable()->after('reminder_client_push_sent_at');
        });
    }

    public function down(): void
    {
        Schema::table('appointments', function (Blueprint $table) {
            $table->dropColumn(['reminder_client_push_sent_at', 'reminder_barber_push_sent_at']);
        });
    }
};
