import{_ as b,u,a as v,o as _,c as f,b as e,t as d,d as l,h as t,D as m,i as r,l as n,A as i,j as g}from"./index-DvnNK70g.js";import{L as h}from"./LogoutButton-DrzvB6m_.js";const S={class:"barber-layout"},B={class:"barber-header"},C={class:"header-mode"},L={class:"header-title"},$={class:"barber-header-actions"},k={class:"barber-content"},y={class:"bottom-nav"},A={__name:"BarberLayout",setup(N){const p=u(),c=v();return(s,a)=>{const o=g("router-link");return _(),f("div",S,[e("header",B,[e("div",null,[e("div",C,"Modo "+d(l(c).staffLabelSingular),1),e("div",L,d(l(p).user?.name?.toUpperCase()||l(c).staffLabelSingular.toUpperCase()),1)]),e("div",$,[t(h),a[0]||(a[0]=e("span",{class:"badge badge-green"},"Activo",-1))])]),e("main",k,[m(s.$slots,"default",{},void 0)]),e("nav",y,[t(o,{to:"/barber/agenda",class:i(["nav-item",{active:s.$route.path==="/barber/agenda"}])},{default:r(()=>[...a[1]||(a[1]=[e("span",{class:"nav-icon"},"📋",-1),n(" Turnos ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/clientes",class:i(["nav-item",{active:s.$route.path==="/barber/clientes"}])},{default:r(()=>[...a[2]||(a[2]=[e("span",{class:"nav-icon"},"👥",-1),n(" Clientes ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/servicio-especial",class:i(["nav-item",{active:s.$route.path==="/barber/servicio-especial"}])},{default:r(()=>[...a[3]||(a[3]=[e("span",{class:"nav-icon"},"✂️",-1),n(" Especial ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/perfil",class:i(["nav-item",{active:s.$route.path==="/barber/perfil"}])},{default:r(()=>[...a[4]||(a[4]=[e("span",{class:"nav-icon"},"👤",-1),n(" Perfil ",-1)])]),_:1},8,["class"])])])}}},E=b(A,[["__scopeId","data-v-c14ea11c"]]);export{E as B};
