.settings-card[data-v-2dafb53f]{padding:20px;max-width:720px}.wa-copy-head[data-v-2dafb53f]{margin-bottom:16px;max-width:720px}.wa-subnav[data-v-2dafb53f]{font-size:.9rem;margin-bottom:4px}.wa-subnav__link[data-v-2dafb53f]{color:#7ab8ff;text-decoration:none}.wa-subnav__link[data-v-2dafb53f]:hover{text-decoration:underline}.wa-subnav__sep[data-v-2dafb53f]{margin:0 6px;color:var(--text-muted)}.wa-subnav__here[data-v-2dafb53f]{font-weight:600;color:var(--text-main)}.wa-copy-lead[data-v-2dafb53f]{margin:0;font-size:.92rem;line-height:1.5;color:var(--text-muted)}.wa-copy-loading[data-v-2dafb53f]{margin:0;color:var(--text-muted)}.wa-copy-switch[data-v-2dafb53f]{margin-bottom:0}.section-title[data-v-2dafb53f]{margin:20px 0 8px;font-size:1rem;font-weight:700;color:var(--text-main)}.field-hint[data-v-2dafb53f]{font-size:.8rem;color:var(--text-muted);margin:0 0 8px;line-height:1.4}.field-hint code[data-v-2dafb53f]{font-size:.85em;padding:1px 4px;background:#00000040;border-radius:4px}.wa-textarea-copy[data-v-2dafb53f]{width:100%;min-height:72px;resize:vertical;font-size:.88rem;line-height:1.45}.wa-copy-actions[data-v-2dafb53f]{margin:16px 0 12px}.success-text[data-v-2dafb53f]{margin:6px 0 12px;color:var(--success)}
