import{_ as n,o as s,c as o,a}from"./index-DcK5qV02.js";const c={},r={class:"spinner-container"};function t(i,e){return s(),o("div",r,[...e[0]||(e[0]=[a("div",{class:"spinner"},null,-1)])])}const d=n(c,[["render",t],["__scopeId","data-v-0e4c7a5f"]]);export{d as L};
