.client-layout[data-v-a2c6e691]{min-height:100vh;min-height:100dvh;display:flex;flex-direction:column;background:var(--bg-device)}.client-header[data-v-a2c6e691]{padding:16px 20px;padding-top:calc(16px + env(safe-area-inset-top));background:#000000f2;border-bottom:1px solid var(--border);display:flex;justify-content:space-between;align-items:center;gap:12px;position:sticky;top:0;z-index:10}.client-header__brand[data-v-a2c6e691]{min-width:0}.header-mode[data-v-a2c6e691]{font-size:10px;color:var(--gold);text-transform:uppercase;letter-spacing:.08em;font-weight:700}.client-header .header-title[data-v-a2c6e691]{font-size:18px;font-weight:800;color:var(--gold);letter-spacing:.06em;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:min(200px,52vw);line-height:1.25;margin-top:2px}.header-business[data-v-a2c6e691]{font-size:11px;color:var(--text-muted);margin-top:2px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:min(220px,58vw)}.client-header__actions[data-v-a2c6e691]{display:flex;align-items:center;gap:10px;flex-shrink:0}.header-avatar[data-v-a2c6e691]{width:35px;height:35px;background:var(--bg-card);border-radius:50%;display:flex;align-items:center;justify-content:center;border:1px solid var(--gold);font-size:12px;font-weight:600;color:var(--gold)}.client-content[data-v-a2c6e691]{padding:16px 20px 24px;overflow-y:auto;overflow-x:hidden;flex:1;-webkit-overflow-scrolling:touch}.client-shell[data-v-a2c6e691]{width:100%;max-width:min(28rem,100%);margin:0 auto}.client-layout--wa-fab .client-content[data-v-a2c6e691]{padding-bottom:calc(24px + 5.5rem)}.client-wa-fab[data-v-a2c6e691]{position:fixed;right:calc(16px + env(safe-area-inset-right,0px));bottom:calc(72px + env(safe-area-inset-bottom,0px));z-index:20;display:flex;align-items:center;justify-content:center;width:56px;height:56px;border-radius:50%;background:#25d366;color:#fff;text-decoration:none;box-shadow:0 4px 18px #00000073,0 0 0 1px #ffffff1f;transition:transform .15s ease,box-shadow .15s ease;touch-action:manipulation}.client-wa-fab[data-v-a2c6e691]:active{transform:scale(.96)}.client-wa-fab__icon[data-v-a2c6e691]{font-size:1.55rem;line-height:1}
