<?php

declare(strict_types=1);

namespace SpomkyLabs\Pki\CryptoTypes\AlgorithmIdentifier\Feature;

/**
 * Algorithm identifier for asymmetric cryptography algorithms.
 */
interface AsymmetricCryptoAlgorithmIdentifier extends AlgorithmIdentifierType
{
}
