.client-layout[data-v-d6f4e0fc]{min-height:100vh;min-height:100dvh;display:flex;flex-direction:column;background:var(--bg-device)}.client-header[data-v-d6f4e0fc]{padding:14px 18px;padding-top:calc(12px + env(safe-area-inset-top));background:#000000f2;border-bottom:1px solid var(--border);display:flex;justify-content:space-between;align-items:center;gap:12px;position:sticky;top:0;z-index:10}.client-header__brand[data-v-d6f4e0fc]{min-width:0}.header-mode[data-v-d6f4e0fc]{font-size:10px;color:var(--gold);text-transform:uppercase;letter-spacing:.08em;font-weight:700}.client-header .header-title[data-v-d6f4e0fc]{font-size:17px;font-weight:800;color:var(--gold);letter-spacing:.06em;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:min(200px,52vw);line-height:1.25;margin-top:2px}.header-business[data-v-d6f4e0fc]{font-size:11px;color:var(--text-muted);margin-top:2px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:min(220px,58vw)}.client-header__actions[data-v-d6f4e0fc]{display:flex;align-items:center;gap:10px;flex-shrink:0}.header-avatar[data-v-d6f4e0fc]{width:36px;height:36px;background:var(--bg-card);border-radius:50%;display:flex;align-items:center;justify-content:center;border:1px solid color-mix(in srgb,var(--gold) 45%,var(--border));font-size:11px;font-weight:700;color:var(--gold)}.client-content[data-v-d6f4e0fc]{padding:16px 16px 12px;overflow-y:auto;overflow-x:hidden;flex:1;-webkit-overflow-scrolling:touch}.client-shell[data-v-d6f4e0fc]{width:100%;max-width:min(28rem,100%);margin:0 auto}.client-layout--wa-fab .client-content[data-v-d6f4e0fc]{padding-bottom:calc(12px + 5.5rem)}.client-bottom-nav[data-v-d6f4e0fc]{display:flex;align-items:stretch;justify-content:space-between;gap:4px;margin:0 10px calc(10px + env(safe-area-inset-bottom));padding:6px 8px;flex-shrink:0;background:color-mix(in srgb,var(--bg-card) 88%,transparent);border:1px solid color-mix(in srgb,var(--border) 85%,transparent);border-radius:999px;box-shadow:0 8px 32px #00000059;backdrop-filter:blur(14px);-webkit-backdrop-filter:blur(14px)}.client-nav-item[data-v-d6f4e0fc]{flex:1;min-width:0;display:flex;flex-direction:column;align-items:center;justify-content:center;gap:3px;padding:8px 2px;border-radius:999px;text-decoration:none;color:var(--text-muted);font-size:9px;font-weight:700;letter-spacing:.02em;text-transform:uppercase;transition:color .18s ease,background .18s ease;touch-action:manipulation}.client-nav-item__icon[data-v-d6f4e0fc]{display:flex;align-items:center;justify-content:center;width:22px;height:22px;opacity:.88}.client-nav-item__icon[data-v-d6f4e0fc] svg{width:22px;height:22px}.client-nav-item.active[data-v-d6f4e0fc]{color:var(--gold);background:color-mix(in srgb,var(--gold) 12%,transparent)}.client-nav-item.active .client-nav-item__icon[data-v-d6f4e0fc]{opacity:1}.client-nav-item[data-v-d6f4e0fc]:active:not(.active){color:var(--text-main)}.client-wa-fab[data-v-d6f4e0fc]{position:fixed;right:calc(14px + env(safe-area-inset-right,0px));bottom:calc(88px + env(safe-area-inset-bottom,0px));z-index:20;display:flex;align-items:center;justify-content:center;width:54px;height:54px;border-radius:50%;background:#25d366;color:#fff;text-decoration:none;box-shadow:0 4px 18px #00000073,0 0 0 1px #ffffff1f;transition:transform .15s ease,box-shadow .15s ease;touch-action:manipulation}.client-wa-fab[data-v-d6f4e0fc]:active{transform:scale(.96)}.client-wa-fab__icon[data-v-d6f4e0fc]{font-size:1.45rem;line-height:1}@media(min-width:480px){.client-bottom-nav[data-v-d6f4e0fc]{max-width:min(28rem,calc(100% - 20px));margin-left:auto;margin-right:auto}}
