import{_ as r,o,f as c,c as s,b as a,g as d,t as m,L as u,C as i,K as f}from"./index-BpX-8PVd.js";const b={key:0,class:"modal-sheet-handle","aria-hidden":"true"},y={key:1,class:"modal-title"},h={class:"modal-body"},v={key:2,class:"modal-actions"},k={__name:"ModalDialog",props:{show:Boolean,title:String,wide:{type:Boolean,default:!1},extraWide:{type:Boolean,default:!1},showActions:{type:Boolean,default:!0},sheet:{type:Boolean,default:!1}},emits:["close","confirm"],setup(e){return(l,t)=>(o(),c(f,{to:"body"},[e.show?(o(),s("div",{key:0,class:i(["modal-overlay",{"modal-overlay--sheet":e.sheet}])},[a("div",{class:"modal-backdrop",onClick:t[0]||(t[0]=n=>l.$emit("close"))}),a("div",{class:i(["modal-card",{"modal-card--wide":e.wide,"modal-card--extra-wide":e.extraWide,"modal-card--sheet":e.sheet}]),role:"dialog","aria-modal":"true"},[e.sheet?(o(),s("div",b)):d("",!0),e.title?(o(),s("h3",y,m(e.title),1)):d("",!0),a("div",h,[u(l.$slots,"default",{},void 0)]),e.showActions?(o(),s("div",v,[a("button",{type:"button",onClick:t[1]||(t[1]=n=>l.$emit("close")),class:"btn btn-outline btn-sm"}," Cancelar "),a("button",{type:"button",onClick:t[2]||(t[2]=n=>l.$emit("confirm")),class:"btn btn-gold btn-sm"}," Confirmar ")])):d("",!0)],2)],2)):d("",!0)]))}},w=r(k,[["__scopeId","data-v-cb858ffb"]]);export{w as M};
