.dashboard-header[data-v-221fbd87]{display:flex;justify-content:space-between;align-items:center;margin-bottom:30px}.date-row[data-v-221fbd87]{margin-bottom:24px;display:grid;grid-template-columns:1fr auto;gap:24px;align-items:start}@media(max-width:900px){.date-row[data-v-221fbd87]{grid-template-columns:1fr}}.date-row-inner .title-sm[data-v-221fbd87]{display:block;margin-bottom:10px}.dashboard-cal[data-v-221fbd87]{max-width:420px}.filter-row[data-v-221fbd87]{min-width:200px}.filter-row .title-sm[data-v-221fbd87]{display:block;margin-bottom:8px}.filter-select[data-v-221fbd87]{min-width:200px}.stats-grid[data-v-221fbd87]{display:grid;grid-template-columns:repeat(3,1fr);gap:20px;margin-bottom:30px}.stat-card[data-v-221fbd87]{margin:0}.stat-label[data-v-221fbd87]{color:var(--text-muted);font-size:12px;text-transform:uppercase}.stat-value[data-v-221fbd87]{font-size:28px;font-weight:700;color:var(--gold);margin-top:5px}.stat-value.stat-success[data-v-221fbd87]{color:var(--success)}.table-card[data-v-221fbd87]{margin:0}.no-data[data-v-221fbd87]{color:var(--text-muted);padding:40px;text-align:center}.table[data-v-221fbd87]{width:100%;border-collapse:collapse}.table th[data-v-221fbd87],.table td[data-v-221fbd87]{padding:15px;text-align:left;border-bottom:1px solid var(--border)}.table th[data-v-221fbd87]{color:var(--text-muted);font-size:12px;text-transform:uppercase}@media(max-width:768px){.dashboard-header[data-v-221fbd87]{flex-direction:column;align-items:stretch;gap:16px}.dashboard-header .btn[data-v-221fbd87]{width:100%}.stats-grid[data-v-221fbd87]{grid-template-columns:1fr}.table-card[data-v-221fbd87]{overflow-x:auto;-webkit-overflow-scrolling:touch}.table[data-v-221fbd87]{min-width:960px}}.hint-muted[data-v-221fbd87]{font-size:.82rem;color:var(--text-muted);margin:8px 0 0}.th-actions[data-v-221fbd87],.td-actions[data-v-221fbd87]{white-space:nowrap;vertical-align:middle}.action-btns[data-v-221fbd87]{display:flex;flex-wrap:wrap;gap:8px}.action-btn-inline[data-v-221fbd87]{min-height:36px;padding:6px 12px;font-size:12px}.actions-dash[data-v-221fbd87]{color:var(--text-muted)}.badge-pending[data-v-221fbd87]{background:#e67e2233;color:#e67e22}.badge-cancelled[data-v-221fbd87]{background:#78787840;color:var(--text-muted)}
