import{_ as i,o as s,f as r,c as l,b as o,t as c,g as n,H as m,C as u,T as b}from"./index-CUYQqFVq.js";const f={key:0,class:"modal-overlay"},y={key:0,class:"modal-title"},k={class:"modal-body"},v={key:1,class:"modal-actions"},w={__name:"ModalDialog",props:{show:Boolean,title:String,wide:{type:Boolean,default:!1},extraWide:{type:Boolean,default:!1},showActions:{type:Boolean,default:!0}},emits:["close","confirm"],setup(e){return(a,t)=>(s(),r(b,{to:"body"},[e.show?(s(),l("div",f,[o("div",{class:"modal-backdrop",onClick:t[0]||(t[0]=d=>a.$emit("close"))}),o("div",{class:u(["modal-card",{"modal-card--wide":e.wide,"modal-card--extra-wide":e.extraWide}]),role:"dialog","aria-modal":"true"},[e.title?(s(),l("h3",y,c(e.title),1)):n("",!0),o("div",k,[m(a.$slots,"default",{},void 0)]),e.showActions?(s(),l("div",v,[o("button",{type:"button",onClick:t[1]||(t[1]=d=>a.$emit("close")),class:"btn btn-outline btn-sm"}," Cancelar "),o("button",{type:"button",onClick:t[2]||(t[2]=d=>a.$emit("confirm")),class:"btn btn-gold btn-sm"}," Confirmar ")])):n("",!0)],2)])):n("",!0)]))}},C=i(w,[["__scopeId","data-v-f34b211b"]]);export{C as M};
