import{B as t}from"./index-DcK5qV02.js";const i={login(e,r){return t.post("/auth/login",{email:e,password:r})},register(e){return t.post("/auth/register",e)},logout(){return t.post("/auth/logout")},me(){return t.get("/auth/me")},changePassword(e,r){return t.post("/auth/change-password",{current_password:e,new_password:r})}},s={getServices(){return t.get("/public/services")},getBarbers(){return t.get("/public/barbers")},getSlots(e,r,n){return t.get("/public/slots",{params:{barber_id:e,service_id:r,date:n}})}},p={getAppointments(){return t.get("/client/appointments")},getNextAppointment(){return t.get("/client/appointments/next")},createAppointment(e){return t.post("/client/appointments",e)},cancelAppointment(e){return t.delete(`/client/appointments/${e}`)},getExpediente(){return t.get("/client/expediente")},getSlots(e,r,n){return t.get("/shared/slots",{params:{barber_id:e,service_id:r,date:n}})}},o={getServices(){return t.get("/admin/services")},createService(e){return t.post("/admin/services",e)},updateService(e,r){return t.put(`/admin/services/${e}`,r)},deleteService(e){return t.delete(`/admin/services/${e}`)},getBarbers(){return t.get("/admin/barbers")},createBarber(e){return t.post("/admin/barbers",e)},updateBarber(e,r){return t.put(`/admin/barbers/${e}`,r)},deleteBarber(e){return t.delete(`/admin/barbers/${e}`)},getMonitor(e){return t.get("/admin/monitor",{params:{date:e}})},getClients(){return t.get("/admin/clients")},getClientsList(){return t.get("/admin/clients")},getClientExpediente(e){return t.get(`/admin/clients/${e}/expediente`)},deletePhoto(e){return t.delete(`/admin/photos/${e}`)},createClient(e){return t.post("/admin/clients",e)},updateClient(e,r){return t.put(`/admin/clients/${e}`,r)},deleteClient(e){return t.delete(`/admin/clients/${e}`)},activateClient(e){return t.put(`/admin/clients/${e}`,{is_active:!0})},createAppointment(e){return t.post("/admin/appointments",e)}},u={getAppointment(e){return t.get(`/barber/appointments/${e}`)},getAgenda(e){return t.get("/barber/agenda",{params:{date:e}})},completeAppointment(e){return t.patch(`/barber/appointments/${e}/complete`)},getPhotos(e){return t.get(`/barber/appointments/${e}/photos`)},uploadPhoto(e,r){return t.post(`/barber/appointments/${e}/photos`,r,{headers:{"Content-Type":"multipart/form-data"}})},deletePhoto(e,r){return t.delete(`/barber/appointments/${e}/photos/${r}`)},getCustomServiceLogs(e){return t.get("/barber/custom-services",{params:{date:e}})},createCustomServiceLog(e){return t.post("/barber/custom-services",e)}};export{i as a,u as b,p as c,o as d,s as p};
