import{_ as p,u,a as v,o as f,c as m,b as a,t as b,d as l,h as t,L as _,i as r,l as n,D as i,j as g}from"./index-BJRAPt-R.js";import{L as h}from"./LogoutButton-DFUoesFS.js";const S={class:"barber-layout"},B={class:"barber-header"},L={class:"header-mode"},C={class:"header-title"},$={class:"barber-header-actions"},k={class:"barber-content"},y={class:"bottom-nav"},A={__name:"BarberLayout",setup(N){const d=u(),c=v();return(s,e)=>{const o=g("router-link");return f(),m("div",S,[a("header",B,[a("div",null,[a("div",L,"Modo "+b(l(c).staffLabelSingular),1),a("div",C,b(l(d).user?.name?.toUpperCase()||l(c).staffLabelSingular.toUpperCase()),1)]),a("div",$,[t(h),e[0]||(e[0]=a("span",{class:"badge badge-green"},"Activo",-1))])]),a("main",k,[_(s.$slots,"default",{},void 0)]),a("nav",y,[t(o,{to:"/barber/agenda",class:i(["nav-item",{active:s.$route.path==="/barber/agenda"}])},{default:r(()=>[...e[1]||(e[1]=[a("span",{class:"nav-icon"},"📋",-1),n(" Turnos ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/inbox",class:i(["nav-item",{active:s.$route.path==="/barber/inbox"}])},{default:r(()=>[...e[2]||(e[2]=[a("span",{class:"nav-icon"},"📥",-1),n(" Por confirmar ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/clientes",class:i(["nav-item",{active:s.$route.path==="/barber/clientes"}])},{default:r(()=>[...e[3]||(e[3]=[a("span",{class:"nav-icon"},"👥",-1),n(" Clientes ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/servicio-especial",class:i(["nav-item",{active:s.$route.path==="/barber/servicio-especial"}])},{default:r(()=>[...e[4]||(e[4]=[a("span",{class:"nav-icon"},"✂️",-1),n(" Especial ",-1)])]),_:1},8,["class"]),t(o,{to:"/barber/perfil",class:i(["nav-item",{active:s.$route.path==="/barber/perfil"}])},{default:r(()=>[...e[5]||(e[5]=[a("span",{class:"nav-icon"},"👤",-1),n(" Perfil ",-1)])]),_:1},8,["class"])])])}}},E=p(A,[["__scopeId","data-v-31f5b474"]]);export{E as B};
