import{_ as v,a as f,o as r,c as m,b as e,A as t,t as b,d as $,h as i,i as d,l as u,D as k,r as C,j as A}from"./index-DBt8KrOe.js";import{L as y}from"./LogoutButton-DkssZJjE.js";const S={class:"admin-layout"},g={key:0},B={key:1},j={class:"sidebar-brand"},L={class:"sidebar-nav"},N={class:"sidebar-footer"},w={class:"admin-main"},V={__name:"AdminLayout",setup(D){const a=C(!1),p=f();return(n,s)=>{const o=A("router-link");return r(),m("div",S,[e("button",{class:"sidebar-toggle",onClick:s[0]||(s[0]=l=>a.value=!a.value),"aria-label":"Menú"},[a.value?(r(),m("span",B,"✕")):(r(),m("span",g,"☰"))]),e("div",{class:t(["sidebar-backdrop",{open:a.value}]),onClick:s[1]||(s[1]=l=>a.value=!1)},null,2),e("aside",{class:t(["sidebar",{open:a.value}])},[e("div",j,b($(p).businessName),1),e("nav",L,[i(o,{to:"/admin",class:t(["menu-item",{active:n.$route.path==="/admin"}]),onClick:s[2]||(s[2]=l=>a.value=!1)},{default:d(()=>[...s[9]||(s[9]=[e("span",null,"📊",-1),u(" Monitor Hoy ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/barbers",class:t(["menu-item",{active:n.$route.path==="/admin/barbers"}]),onClick:s[3]||(s[3]=l=>a.value=!1)},{default:d(()=>[...s[10]||(s[10]=[e("span",null,"💈",-1),u(" Barberos ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/services",class:t(["menu-item",{active:n.$route.path==="/admin/services"}]),onClick:s[4]||(s[4]=l=>a.value=!1)},{default:d(()=>[...s[11]||(s[11]=[e("span",null,"✂️",-1),u(" Servicios ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/clients",class:t(["menu-item",{active:n.$route.path==="/admin/clients"}]),onClick:s[5]||(s[5]=l=>a.value=!1)},{default:d(()=>[...s[12]||(s[12]=[e("span",null,"👥",-1),u(" Clientes ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/admins",class:t(["menu-item",{active:n.$route.path==="/admin/admins"}]),onClick:s[6]||(s[6]=l=>a.value=!1)},{default:d(()=>[...s[13]||(s[13]=[e("span",null,"🛡️",-1),u(" Administradores ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/whatsapp",class:t(["menu-item",{active:n.$route.path==="/admin/whatsapp"}]),onClick:s[7]||(s[7]=l=>a.value=!1)},{default:d(()=>[...s[14]||(s[14]=[e("span",null,"💬",-1),u(" WhatsApp ",-1)])]),_:1},8,["class"]),i(o,{to:"/admin/ajustes",class:t(["menu-item",{active:n.$route.path==="/admin/ajustes"}]),onClick:s[8]||(s[8]=l=>a.value=!1)},{default:d(()=>[...s[15]||(s[15]=[e("span",null,"⚙️",-1),u(" Ajustes App ",-1)])]),_:1},8,["class"])]),e("div",N,[i(y)])],2),e("main",w,[k(n.$slots,"default",{},void 0)])])}}},E=v(V,[["__scopeId","data-v-3e9abd6b"]]);export{E as A};
