@props(['message'])

<div class="bg-white/[2%] border border-neutral-200 dark:border-neutral-800 rounded-md w-full p-5 uppercase text-sm text-center font-mono shadow-xs text-neutral-600 dark:text-neutral-400">
    <span class="text-neutral-400 dark:text-neutral-600">// </span>{{ $message }}
</div>
