import{_ as b,a as $,o as v,c as f,b as e,A as t,t as m,d as p,h as l,i as d,l as u,D as k,r as C,j as A}from"./index-CZigIl_A.js";import{L as S}from"./LogoutButton-DatG4Ebf.js";const y={class:"admin-layout"},g={key:0},L={key:1},c={class:"sidebar-brand"},j={class:"sidebar-nav"},B={"aria-hidden":"true"},N={"aria-hidden":"true"},w={class:"sidebar-footer"},I={class:"admin-main"},V={__name:"AdminLayout",setup(D){const a=C(!1),r=$();return(n,s)=>{const o=A("router-link");return v(),f("div",y,[e("button",{class:"sidebar-toggle",onClick:s[0]||(s[0]=i=>a.value=!a.value),"aria-label":"Menú"},[a.value?(v(),f("span",L,"✕")):(v(),f("span",g,"☰"))]),e("div",{class:t(["sidebar-backdrop",{open:a.value}]),onClick:s[1]||(s[1]=i=>a.value=!1)},null,2),e("aside",{class:t(["sidebar",{open:a.value}])},[e("div",c,m(p(r).businessName),1),e("nav",j,[l(o,{to:"/admin",class:t(["menu-item",{active:n.$route.path==="/admin"}]),onClick:s[2]||(s[2]=i=>a.value=!1)},{default:d(()=>[...s[10]||(s[10]=[e("span",null,"📊",-1),u(" Monitor Hoy ",-1)])]),_:1},8,["class"]),l(o,{to:"/admin/inbox",class:t(["menu-item",{active:n.$route.path==="/admin/inbox"}]),onClick:s[3]||(s[3]=i=>a.value=!1)},{default:d(()=>[...s[11]||(s[11]=[e("span",null,"📥",-1),u(" Por confirmar ",-1)])]),_:1},8,["class"]),l(o,{to:"/admin/barbers",class:t(["menu-item",{active:n.$route.path==="/admin/barbers"}]),onClick:s[4]||(s[4]=i=>a.value=!1)},{default:d(()=>[e("span",B,m(p(r).menuIconStaff),1),u(" "+m(p(r).staffLabelPlural),1)]),_:1},8,["class"]),l(o,{to:"/admin/services",class:t(["menu-item",{active:n.$route.path==="/admin/services"}]),onClick:s[5]||(s[5]=i=>a.value=!1)},{default:d(()=>[e("span",N,m(p(r).menuIconServices),1),s[12]||(s[12]=u(" Servicios ",-1))]),_:1},8,["class"]),l(o,{to:"/admin/clients",class:t(["menu-item",{active:n.$route.path==="/admin/clients"}]),onClick:s[6]||(s[6]=i=>a.value=!1)},{default:d(()=>[...s[13]||(s[13]=[e("span",null,"👥",-1),u(" Clientes ",-1)])]),_:1},8,["class"]),l(o,{to:"/admin/admins",class:t(["menu-item",{active:n.$route.path==="/admin/admins"}]),onClick:s[7]||(s[7]=i=>a.value=!1)},{default:d(()=>[...s[14]||(s[14]=[e("span",null,"🛡️",-1),u(" Administradores ",-1)])]),_:1},8,["class"]),l(o,{to:"/admin/whatsapp",class:t(["menu-item",{active:n.$route.path==="/admin/whatsapp"}]),onClick:s[8]||(s[8]=i=>a.value=!1)},{default:d(()=>[...s[15]||(s[15]=[e("span",null,"💬",-1),u(" WhatsApp ",-1)])]),_:1},8,["class"]),l(o,{to:"/admin/ajustes",class:t(["menu-item",{active:n.$route.path==="/admin/ajustes"}]),onClick:s[9]||(s[9]=i=>a.value=!1)},{default:d(()=>[...s[16]||(s[16]=[e("span",null,"⚙️",-1),u(" Ajustes App ",-1)])]),_:1},8,["class"])]),e("div",w,[l(S)])],2),e("main",I,[k(n.$slots,"default",{},void 0)])])}}},z=b(V,[["__scopeId","data-v-9ae7c4e8"]]);export{z as A};
