import{_ as p,u as _,a as m,o as v,c as f,b as t,t as c,d as h,h as a,D as b,i as n,l as i,A as l,x as C,j as S}from"./index-FeGPXkd3.js";import{L as k}from"./LogoutButton-B5z6lFv-.js";const g={class:"client-layout"},L={class:"client-header"},y={class:"header-title"},B={class:"header-actions"},N={class:"header-avatar"},$={class:"client-content"},A={class:"bottom-nav"},I={__name:"ClientLayout",setup(V){const r=_(),u=m(),d=C(()=>(r.user?.name||"").split(" ").map(s=>s[0]).join("").substring(0,2).toUpperCase()||"CL");return(e,s)=>{const o=S("router-link");return v(),f("div",g,[t("header",L,[t("div",y,c(h(u).businessName),1),t("div",B,[a(k),t("div",N,c(d.value),1)])]),t("main",$,[b(e.$slots,"default",{},void 0)]),t("nav",A,[a(o,{to:"/client/inicio",class:l(["nav-item",{active:e.$route.path==="/client/inicio"}])},{default:n(()=>[...s[0]||(s[0]=[t("span",{class:"nav-icon"},"🏠",-1),i(" Inicio ",-1)])]),_:1},8,["class"]),a(o,{to:"/client/book",class:l(["nav-item",{active:e.$route.path==="/client/book"}])},{default:n(()=>[...s[1]||(s[1]=[t("span",{class:"nav-icon"},"📅",-1),i(" Reservar ",-1)])]),_:1},8,["class"]),a(o,{to:"/client/perfil",class:l(["nav-item",{active:e.$route.path==="/client/perfil"}])},{default:n(()=>[...s[2]||(s[2]=[t("span",{class:"nav-icon"},"👤",-1),i(" Perfil ",-1)])]),_:1},8,["class"])])])}}},D=p(I,[["__scopeId","data-v-e5fab43b"]]);export{D as C};
