import{_ as d,o as a,f as r,c as l,b as s,t as c,g as n,D as m,A as u,T as f}from"./index-BmPrhdN5.js";const b={key:0,class:"modal-overlay"},k={key:0,class:"modal-title"},v={key:1,class:"modal-actions"},y={__name:"ModalDialog",props:{show:Boolean,title:String,wide:{type:Boolean,default:!1},extraWide:{type:Boolean,default:!1},showActions:{type:Boolean,default:!0}},emits:["close","confirm"],setup(e){return(o,t)=>(a(),r(f,{to:"body"},[e.show?(a(),l("div",b,[s("div",{class:"modal-backdrop",onClick:t[0]||(t[0]=i=>o.$emit("close"))}),s("div",{class:u(["modal-card",{"modal-card--wide":e.wide,"modal-card--extra-wide":e.extraWide}])},[e.title?(a(),l("h3",k,c(e.title),1)):n("",!0),m(o.$slots,"default",{},void 0),e.showActions?(a(),l("div",v,[s("button",{onClick:t[1]||(t[1]=i=>o.$emit("close")),class:"btn btn-outline btn-sm"}," Cancelar "),s("button",{onClick:t[2]||(t[2]=i=>o.$emit("confirm")),class:"btn btn-gold btn-sm"}," Confirmar ")])):n("",!0)],2)])):n("",!0)]))}},B=d(y,[["__scopeId","data-v-7987102e"]]);export{B as M};
