import{_ as c,u as d,o as b,c as p,a,t as u,s as v,x as m,f as r,g as o,j as n,q as l,h as f}from"./index-bWZW5iIe.js";const _={class:"barber-layout"},B={class:"barber-header"},g={class:"header-title"},h={class:"barber-content"},C={class:"bottom-nav"},$={__name:"BarberLayout",setup(k){const i=d();return(s,e)=>{const t=f("router-link");return b(),p("div",_,[a("header",B,[a("div",null,[e[0]||(e[0]=a("div",{class:"header-mode"},"Modo Barbero",-1)),a("div",g,u(v(i).user?.name?.toUpperCase()||"BARBERO"),1)]),e[1]||(e[1]=a("span",{class:"badge badge-green"},"Activo",-1))]),a("main",h,[m(s.$slots,"default",{},void 0)]),a("nav",C,[r(t,{to:"/barber/agenda",class:l(["nav-item",{active:s.$route.path==="/barber/agenda"}])},{default:o(()=>[...e[2]||(e[2]=[a("span",{class:"nav-icon"},"📋",-1),n(" Turnos ",-1)])]),_:1},8,["class"]),r(t,{to:"/barber/clientes",class:l(["nav-item",{active:s.$route.path==="/barber/clientes"}])},{default:o(()=>[...e[3]||(e[3]=[a("span",{class:"nav-icon"},"👥",-1),n(" Clientes ",-1)])]),_:1},8,["class"]),r(t,{to:"/barber/servicio-especial",class:l(["nav-item",{active:s.$route.path==="/barber/servicio-especial"}])},{default:o(()=>[...e[4]||(e[4]=[a("span",{class:"nav-icon"},"✂️",-1),n(" Especial ",-1)])]),_:1},8,["class"]),r(t,{to:"/barber/perfil",class:l(["nav-item",{active:s.$route.path==="/barber/perfil"}])},{default:o(()=>[...e[5]||(e[5]=[a("span",{class:"nav-icon"},"👤",-1),n(" Perfil ",-1)])]),_:1},8,["class"])])])}}},S=c($,[["__scopeId","data-v-ec303bc3"]]);export{S as B};
