import{_ as n,o as s,c as o,b as c}from"./index-BSoZUQlM.js";const r={},a={class:"spinner-container"};function t(i,e){return s(),o("div",a,[...e[0]||(e[0]=[c("div",{class:"spinner"},null,-1)])])}const d=n(r,[["render",t],["__scopeId","data-v-0e4c7a5f"]]);export{d as L};
