<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    public function up(): void
    {
        Schema::table('services', function (Blueprint $table) {
            $table->string('icon', 64)->nullable()->after('name');
        });

        Schema::table('barbers', function (Blueprint $table) {
            $table->string('icon', 64)->nullable()->after('specialty');
        });
    }

    public function down(): void
    {
        Schema::table('services', function (Blueprint $table) {
            $table->dropColumn('icon');
        });

        Schema::table('barbers', function (Blueprint $table) {
            $table->dropColumn('icon');
        });
    }
};
