<?php

declare(strict_types=1);

namespace SpomkyLabs\Pki\X509\AttributeCertificate\Validation\Exception;

use SpomkyLabs\Pki\X509\Exception\X509ValidationException;

final class ACValidationException extends X509ValidationException
{
}
