.client-combobox[data-v-75458bcb]{margin-bottom:0}.client-combobox-wrap[data-v-75458bcb]{position:relative}.client-combobox-list[data-v-75458bcb]{position:absolute;left:0;right:0;top:calc(100% + 4px);z-index:1200;margin:0;padding:6px 0;list-style:none;max-height:240px;overflow-y:auto;background:var(--bg-card, #1e1e28);border:1px solid var(--border, #333);border-radius:8px;box-shadow:0 8px 24px #00000059}.client-combobox-li[data-v-75458bcb]{padding:10px 14px;cursor:pointer;display:flex;flex-direction:column;gap:2px;font-size:14px;color:var(--text, #eee)}.client-combobox-li.muted[data-v-75458bcb]{cursor:default;color:var(--text-muted, #888);font-size:13px}.client-combobox-li.is-active[data-v-75458bcb],.client-combobox-li[data-v-75458bcb]:hover:not(.muted){background:#9a7b1a26}.client-combobox-name[data-v-75458bcb]{font-weight:600}.client-combobox-meta[data-v-75458bcb]{font-size:12px;color:var(--text-muted, #9a9aaa)}
