.client-shell-header[data-v-9ffd315a]{position:sticky;top:0;z-index:90;display:flex;align-items:center;gap:10px;min-height:52px;padding:8px max(12px,env(safe-area-inset-right)) 8px max(12px,env(safe-area-inset-left));padding-top:max(8px,env(safe-area-inset-top));background:color-mix(in srgb,var(--bg-page) 82%,transparent);backdrop-filter:blur(18px);-webkit-backdrop-filter:blur(18px);border-bottom:1px solid color-mix(in srgb,var(--stitch-outline-variant, #4d4635) 22%,transparent);box-shadow:0 4px 30px #0000001f;box-sizing:border-box}.client-shell-header__cluster[data-v-9ffd315a]{position:relative;flex-shrink:0;display:flex;align-items:center;gap:6px}.client-shell-header__left[data-v-9ffd315a]{position:relative}.client-shell-back[data-v-9ffd315a]{width:36px;height:36px;padding:0;border:1px solid var(--border);border-radius:10px;background:color-mix(in srgb,var(--bg-card) 88%,transparent);color:var(--gold);font-size:1.35rem;line-height:1;cursor:pointer;touch-action:manipulation;display:flex;align-items:center;justify-content:center;flex-shrink:0}.client-shell-back[data-v-9ffd315a]:active{background:color-mix(in srgb,var(--gold) 14%,var(--bg-card))}.client-shell-back__icon[data-v-9ffd315a]{display:block;margin-top:-2px;font-weight:300}.client-shell-header__title[data-v-9ffd315a]{flex:1;min-width:0;margin:0;font-size:.95rem;font-weight:800;color:var(--gold);letter-spacing:.02em;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;text-align:center}.client-shell-header__right[data-v-9ffd315a]{flex-shrink:0;min-width:5rem;display:flex;justify-content:flex-end}.client-shell-header__right--spacer[data-v-9ffd315a]{min-width:40px}.client-shell-header__reserve[data-v-9ffd315a]{text-decoration:none;font-weight:700;padding:8px 12px;min-height:40px;display:inline-flex;align-items:center;justify-content:center}.client-avatar-btn[data-v-9ffd315a]{width:40px;height:40px;padding:0;border:1px solid color-mix(in srgb,var(--stitch-outline-variant, #4d4635) 35%,transparent);border-radius:50%;background:var(--bg-card-hover);color:var(--gold);cursor:pointer;touch-action:manipulation;display:flex;align-items:center;justify-content:center;box-shadow:0 1px 6px #00000059}.client-avatar-btn__inner[data-v-9ffd315a]{font-size:.78rem;font-weight:800;letter-spacing:-.02em;line-height:1}.client-account-dropdown[data-v-9ffd315a]{position:absolute;top:calc(100% + 8px);left:0;min-width:200px;padding:6px 0;background:var(--bg-card);border:1px solid var(--border);border-radius:10px;box-shadow:0 8px 28px #00000073;z-index:95}.client-account-dropdown__item[data-v-9ffd315a]{display:block;width:100%;padding:12px 14px;border:none;background:none;text-align:left;font-size:.9rem;color:var(--text-main);text-decoration:none;cursor:pointer;font-family:inherit;touch-action:manipulation}.client-account-dropdown__item[data-v-9ffd315a]:hover,.client-account-dropdown__item[data-v-9ffd315a]:focus-visible{background:color-mix(in srgb,var(--gold) 12%,transparent);outline:none}.client-account-dropdown__item--danger[data-v-9ffd315a]{color:var(--danger, #e85d5d)}.client-account-backdrop[data-v-9ffd315a]{position:fixed;inset:0;z-index:88;background:transparent}.client-wa-fab[data-v-9ffd315a]{position:fixed;right:calc(16px + env(safe-area-inset-right,0px));bottom:calc(20px + env(safe-area-inset-bottom,0px));z-index:95;display:flex;align-items:center;justify-content:center;width:56px;height:56px;border-radius:50%;background:var(--wa-green, #25d366);color:#fff;text-decoration:none;box-shadow:0 4px 18px #00000073,0 0 0 1px #ffffff1f;transition:transform .15s ease,box-shadow .15s ease;touch-action:manipulation}.client-wa-fab[data-v-9ffd315a]:active{transform:scale(.96)}.client-wa-fab__svg[data-v-9ffd315a]{width:28px;height:28px;display:block}@media(min-width:769px){.client-wa-fab[data-v-9ffd315a]{right:calc(24px + env(safe-area-inset-right,0px));bottom:calc(24px + env(safe-area-inset-bottom,0px))}}
