import{_ as e,k as s,u,o as c,c as a}from"./index-BJRAPt-R.js";const r={__name:"LogoutButton",setup(_){const t=s(),o=u();async function n(){await o.logout(),t.push("/login")}return(i,p)=>(c(),a("button",{type:"button",class:"btn btn-outline btn-sm logout-btn-unified",onClick:n}," Cerrar sesión "))}},f=e(r,[["__scopeId","data-v-d17ffc02"]]);export{f as L};
