import{B as s}from"./BarberLayout-B1FpBx_5.js";import{f as a,i as r,o as i,b as e}from"./index-FeGPXkd3.js";import"./LogoutButton-B5z6lFv-.js";const c={__name:"ClientesView",setup(o){return(l,t)=>(i(),a(s,null,{default:r(()=>[...t[0]||(t[0]=[e("h2",{style:{margin:"0 0 20px 0","font-size":"20px"}},"Clientes",-1),e("p",{class:"title-sm",style:{"margin-top":"0"}},"Vista de clientes (próximamente)",-1),e("div",{class:"card"},[e("p",{style:{color:"var(--text-muted)"}},"Esta sección mostrará el historial de clientes atendidos.")],-1)])]),_:1}))}};export{c as default};
