import{A as a}from"./AdminLayout-XLUbL7fl.js";import{d as s,g as r,o,a as e}from"./index-DcK5qV02.js";const l={__name:"AjustesView",setup(i){return(n,t)=>(o(),s(a,null,{default:r(()=>[...t[0]||(t[0]=[e("h2",{style:{margin:"0 0 20px 0"}},"Ajustes App",-1),e("p",{class:"title-sm",style:{"margin-top":"0"}},"Configuración de la aplicación",-1),e("div",{class:"card"},[e("p",{style:{color:"var(--text-muted)"}},"Opciones de configuración (próximamente).")],-1)])]),_:1}))}};export{l as default};
