import{_ as p,u as v,o as m,c as _,a as s,t as h,x as f,f as o,g as n,j as i,q as l,p as C,h as g,i as k}from"./index-oOcXnyQ9.js";const b={class:"client-layout"},S={class:"client-header"},y={class:"header-actions"},B={class:"header-avatar"},L={class:"client-content"},$={class:"bottom-nav"},I={__name:"ClientLayout",setup(N){const c=k(),r=v(),u=async()=>{await r.logout(),c.push("/login")},d=C(()=>(r.user?.name||"").split(" ").map(t=>t[0]).join("").substring(0,2).toUpperCase()||"CL");return(e,t)=>{const a=g("router-link");return m(),_("div",b,[s("header",S,[t[0]||(t[0]=s("div",{class:"header-title"},"Barbería San Ramón",-1)),s("div",y,[s("button",{onClick:u,class:"header-logout",title:"Cerrar sesión"},"Salir"),s("div",B,h(d.value),1)])]),s("main",L,[f(e.$slots,"default",{},void 0)]),s("nav",$,[o(a,{to:"/client/inicio",class:l(["nav-item",{active:e.$route.path==="/client/inicio"}])},{default:n(()=>[...t[1]||(t[1]=[s("span",{class:"nav-icon"},"🏠",-1),i(" Inicio ",-1)])]),_:1},8,["class"]),o(a,{to:"/client/book",class:l(["nav-item",{active:e.$route.path==="/client/book"}])},{default:n(()=>[...t[2]||(t[2]=[s("span",{class:"nav-icon"},"📅",-1),i(" Reservar ",-1)])]),_:1},8,["class"]),o(a,{to:"/client/perfil",class:l(["nav-item",{active:e.$route.path==="/client/perfil"}])},{default:n(()=>[...t[3]||(t[3]=[s("span",{class:"nav-icon"},"👤",-1),i(" Perfil ",-1)])]),_:1},8,["class"])])])}}},V=p(I,[["__scopeId","data-v-5ef5431d"]]);export{V as C};
